@@ -11,7 +11,7 @@ our $VERSION;
 # $VERSION declaration must stay up here, ahead of any other package
 # declarations, as to not confuse various modules attempting to determine
 # this ones version, whether that be s.c.o. or Module::Metadata, etc
-$VERSION = '0.082700_01';
+$VERSION = '0.082700_03';
 
 $VERSION = eval $VERSION if $VERSION =~ /_/; # numify for warning-free dev releases
 
@@ -396,6 +396,8 @@ Grant Street Group L<http://www.grantstreet.com/>
 
 groditi: Guillermo Roditi <groditi@cpan.org>
 
+guacamole: Fred Steinberg <fred.steinberg@gmail.com>
+
 Haarg: Graham Knop <haarg@haarg.org>
 
 hobbs: Andrew Rodland <arodland@cpan.org>
@@ -428,6 +430,8 @@ jnapiorkowski: John Napiorkowski <jjn1056@yahoo.com>
 
 jon: Jon Schutz <jjschutz@cpan.org>
 
+Joe Carlson <jwcarlson@lbl.gov>
+
 jshirley: J. Shirley <jshirley@gmail.com>
 
 kaare: Kaare Rasmussen
