@@ -23,30 +23,49 @@ sub has_one {
 sub _has_one {
   my ($class, $join_type, $rel, $f_class, $cond, $attrs) = @_;
   unless (ref $cond) {
-    $class->ensure_class_loaded($f_class);
+    my $pri = $class->result_source_instance->_single_pri_col_or_die;
 
-    my $pri = $class->_get_primary_key;
-
-    $class->throw_exception(
-      "might_have/has_one needs a primary key  to infer a join; ".
-      "${class} has none"
-    ) if !defined $pri && (!defined $cond || !length $cond);
-
-    my $f_class_loaded = try { $f_class->columns };
-    my ($f_key,$too_many,$guess);
+    my ($f_key,$guess,$f_rsrc);
     if (defined $cond && length $cond) {
       $f_key = $cond;
       $guess = "caller specified foreign key '$f_key'";
-    } elsif ($f_class_loaded && $f_class->has_column($rel)) {
-      $f_key = $rel;
-      $guess = "using given relationship '$rel' for foreign key";
-    } else {
-      $f_key = $class->_get_primary_key($f_class);
-      $guess = "using primary key of foreign class for foreign key";
     }
-    $class->throw_exception(
-      "No such column ${f_key} on foreign class ${f_class} ($guess)"
-    ) if $f_class_loaded && !$f_class->has_column($f_key);
+    else {
+      # at this point we need to load the foreigner, expensive or not
+      $class->ensure_class_loaded($f_class);
+
+      $f_rsrc = try {
+        my $r = $f_class->result_source_instance;
+        die "There got to be some columns by now... (exception caught and rewritten by catch below)"
+          unless $r->columns;
+        $r;
+      }
+      catch {
+        $class->throw_exception(
+          "Foreign class '$f_class' does not seem to be a Result class "
+        . "(or it simply did not load entirely due to a circular relation chain)"
+        );
+      };
+
+      if ($f_rsrc->has_column($rel)) {
+        $f_key = $rel;
+        $guess = "using given relationship name '$rel' as foreign key column name";
+      }
+      else {
+        $f_key = $f_rsrc->_single_pri_col_or_die;
+        $guess = "using primary key of foreign class for foreign key";
+      }
+    }
+
+# FIXME - this check needs to be moved to schema-composition time...
+#    # only perform checks if the far side was not preloaded above *AND*
+#    # appears to have been loaded by something else (has a rsrc_instance)
+#    if (! $f_rsrc and $f_rsrc = try { $f_class->result_source_instance }) {
+#      $class->throw_exception(
+#        "No such column '$f_key' on foreign class ${f_class} ($guess)"
+#      ) if !$f_rsrc->has_column($f_key);
+#    }
+
     $cond = { "foreign.${f_key}" => "self.${pri}" };
   }
   $class->_validate_has_one_condition($cond);
@@ -63,21 +82,6 @@ sub _has_one {
   1;
 }
 
-sub _get_primary_key {
-  my ( $class, $target_class ) = @_;
-  $target_class ||= $class;
-  my ($pri, $too_many) = try { $target_class->_pri_cols }
-    catch {
-      $class->throw_exception("Can't infer join condition on ${target_class}: $_");
-    };
-
-  $class->throw_exception(
-    "might_have/has_one can only infer join for a single primary key; ".
-    "${class} has more"
-  ) if $too_many;
-  return $pri;
-}
-
 sub _validate_has_one_condition {
   my ($class, $cond )  = @_;
 
@@ -87,10 +91,11 @@ sub _validate_has_one_condition {
     my $self_id = $cond->{$foreign_id};
 
     # we can ignore a bad $self_id because add_relationship handles this
-    # warning
+    # exception
     return unless $self_id =~ /^self\.(.*)$/;
+
     my $key = $1;
-    $class->throw_exception("Defining rel on ${class} that includes ${key} but no such column defined here yet")
+    $class->throw_exception("Defining rel on ${class} that includes '$key' but no such column defined here yet")
         unless $class->has_column($key);
     my $column_info = $class->column_info($key);
     if ( $column_info->{is_nullable} ) {
